;;; custom-load.el --- automatically extracted custom dependencies

;;; Code:

(autoload 'custom-add-loads "cus-load")

(custom-add-loads 'xwem-selections '("xwem-selections"))
(custom-add-loads 'xwem-sound '("xwem-sound"))
(custom-add-loads 'xwem-theme '("xwem-theme"))
(custom-add-loads 'xwem-mogu '("xwem-mogu"))
(custom-add-loads 'xwem-clgen '("xwem-clgen"))
(custom-add-loads 'xwem-registers '("xwem-register"))
(custom-add-loads 'xwem '("xwem-battery" "xwem-clients" "xwem-clswi" "xwem-desktop" "xwem-faces" "xwem-focus" "xwem-frame" "xwem-holer" "xwem-icons" "xwem-keyboard" "xwem-launcher" "xwem-main" "xwem-manage" "xwem-misc" "xwem-mouse" "xwem-recover" "xwem-root" "xwem-rooticon" "xwem-selections" "xwem-sound" "xwem-strokes" "xwem-tabbing" "xwem-theme" "xwem-time" "xwem-tray" "xwem-weather" "xwem-win" "xwem-worklog"))
(custom-add-loads 'sound '("xwem-sound"))
(custom-add-loads 'xwem-time '("xwem-time"))
(custom-add-loads 'xwem-fullscreen '("xwem-netwm"))
(custom-add-loads 'xwem-pager '("xwem-pager"))
(custom-add-loads 'xwem-special '("xwem-special"))
(custom-add-loads 'applications '("xwem-main"))
(custom-add-loads 'xwem-minibuffer '("xwem-minibuffer"))
(custom-add-loads 'xwem-faces '("xwem-faces"))
(custom-add-loads 'xwem-launcher '("xwem-launcher"))
(custom-add-loads 'xwem-keys '("xwem-keyboard"))
(custom-add-loads 'xwem-clswi '("xwem-clswi"))
(custom-add-loads 'xwem-edmacro '("xwem-edmacro"))
(custom-add-loads 'xwem-rooter '("xwem-rooter"))
(custom-add-loads 'xwem-weather '("xwem-weather"))
(custom-add-loads 'xwem-framei '("xwem-framei"))
(custom-add-loads 'xwem-desktop '("xwem-desktop"))
(custom-add-loads 'xwem-batt '("xwem-battery"))
(custom-add-loads 'xwem-cl '("xwem-clients" "xwem-loaddefs" "xwem-transient"))
(custom-add-loads 'xwem-transient '("xwem-transient"))
(custom-add-loads 'xwem-osd '("xwem-osd"))
(custom-add-loads 'xwem-hooks '("xwem-clients" "xwem-frame" "xwem-keyboard" "xwem-main" "xwem-minibuffer" "xwem-mogu" "xwem-win"))
(custom-add-loads 'xwem-frame '("xwem-frame" "xwem-framei"))
(custom-add-loads 'xwem-rooticon '("xwem-rooticon"))
(custom-add-loads 'xwem-keyboard '("xwem-edmacro" "xwem-keyboard" "xwem-keymacro" "xwem-smartmods"))
(custom-add-loads 'xwem-root '("xwem-root"))
(custom-add-loads 'xwem-holer '("xwem-holer"))
(custom-add-loads 'xwem-win '("xwem-win"))
(custom-add-loads 'xwem-tray '("xwem-pager" "xwem-tray"))
(custom-add-loads 'xwem-modes '("xwem-clgen" "xwem-minibuffer" "xwem-mogu" "xwem-netwm" "xwem-rooter" "xwem-smartmods" "xwem-special" "xwem-vline"))
(custom-add-loads 'xwem-strokes '("xwem-strokes"))
(custom-add-loads 'xwem-tab '("xwem-tabbing"))
(custom-add-loads 'xwem-recover '("xwem-recover"))
(custom-add-loads 'xwem-misc '("xwem-interactive" "xwem-misc" "xwem-osd" "xwem-register" "xwem-report"))
(custom-add-loads 'xwem-worklog '("xwem-worklog"))

;;; custom-load.el ends here
