;;; custom-load.el --- automatically extracted custom dependencies

;;; Code:

(autoload 'custom-add-loads "cus-load")

(custom-add-loads 'latin-unity '("latin-unity-vars" "latin-unity"))
(custom-add-loads 'mule '("latin-unity"))

;;; custom-load.el ends here
